# frozen_string_literal: true

#
# Cookbook:: snu_nginx
# Library:: resource/snu_nginx
#
# Copyright:: 2019, Tyler Technologies
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

require 'chef/resource'

class Chef
  class Resource
    # A single resource to tie together the app, config, and service resources
    # in a common pattern.
    class SnuNginx < Resource
      provides :snu_nginx

      default_action :create

      property :name, String, default: 'nginx'
      property :nginx_name, String, name_property: true

      # Any other properties that should be passed on to the underlying resources.
      property :app_properties, Hash, default: {}
      property :config_properties, Hash, default: {}
      property :conf_d_properties, Hash, default: { general: {}, gzip: {}, logging: {}, ssl: {} }
      property :service_properties, Hash, default: {}

      # To create a full Nginx deployment:
      #   - Install the app
      #   - Write the main nginx.conf
      #   - Write configs for general params, gzip, logging, and ssl
      #   - Start up the service
      action :create do
        with_run_context :root do
          snu_nginx_app do
            new_resource.app_properties.each { |k, v| send(k, v) }
          end

          snu_nginx_config do
            new_resource.config_properties.each { |k, v| send(k, v) }
          end

          %i[general gzip logging ssl].each do |cd|
            snu_nginx_conf_d cd.to_s do
              new_resource.conf_d_properties[cd].each { |k, v| send(k, v) }
            end
          end

          snu_nginx_service do
            new_resource.service_properties.each { |k, v| send(k, v) }
          end
        end
      end

      # To remove a full Nginx deployment:
      #   - Kill the service
      #   - Delete all the child configs
      #   - Delete the main nginx.conf
      #   - Uninstall the app
      action :remove do
        with_run_context :root do
          snu_nginx_service { action %i[stop disable remove] }

          %w[ssl logging gzip general].each do |cd|
            snu_nginx_conf_d(cd) { action :remove }
          end

          snu_nginx_config { action :remove }

          snu_nginx_app { action :remove }
        end
      end
    end
  end
end
